#define MULTICAST

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <fcntl.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <sys/types.h>
#include <sys/time.h>
#include <arpa/inet.h>
#include <unistd.h>
#include <time.h>

#include "api.h"

#define UDP_MAXIMUM_SIZE 65535


int main (int argc, char *argv[]) {

    Byte DevNo = 0;
    int MaxGain = 0;
    int MinGain = 0;
    int completed;
    int len;  
    int sockfd;
    struct sockaddr_in addr;
    unsigned char option_ttl;
    #ifdef ip_mreqn
			struct ip_mreqn mgroup;XXX
    #else
			struct ip_mreq mgroup;
    #endif
    int reuse;
    unsigned int addrlen;
    unsigned char udp_packet[UDP_MAXIMUM_SIZE];
		
    // for supporting IT950x or IT951x
    HandleType handleType;
    Byte Tx_NumOfDev = 0;
    Word ChipType = 0;
    //TxModDriverInfo driverInfo;
    DEVICE_INFO driverInfo;
    int adjustoutputgain = 0; // MinGain: -52, MaxGain: 6. Power Level.
    int outgainvalue;
    Dword dwError = ERR_NO_ERROR;
    MODULATION_PARAM ChannelModulation_Setting;
		
    Dword Freq; // freq out, khz
    Word Bw; // bandwidth, khz
    
    Dword ChannelCapacity = 0;


    Freq = atoi(argv[2])*1000; // частота в mhz
    Bw = 8000;


    memset((char *) &mgroup, 0, sizeof(mgroup));
    mgroup.imr_multiaddr.s_addr = inet_addr(argv[1]); // address
    #ifdef ip_mreqn
      mgroup.imr_address.s_addr = INADDR_ANY;
    #else
      mgroup.imr_interface.s_addr = inet_addr(argv[4]); //ifaddr
    #endif
    memset((char *) &addr, 0, sizeof(addr));
    addr.sin_family = AF_INET;
    addr.sin_port = htons(1234); // default port 1234
    addr.sin_addr.s_addr = inet_addr(argv[1]); // address
    addrlen = sizeof(addr);

    sockfd = socket(AF_INET, SOCK_DGRAM, 0);
    if (sockfd < 0) {
			perror("socket(): error ");
			return 0;
    }
    
		
    reuse = 1;
    if (setsockopt(sockfd, SOL_SOCKET, SO_REUSEADDR, (char *)&reuse, sizeof(reuse)) < 0) {
			perror("setsockopt() SO_REUSEADDR: error ");
    }

    if (bind(sockfd, (struct sockaddr*)&addr, sizeof(addr)) < 0) {
			perror("bind(): error");
			close(sockfd);
			return 0;
    }
    
    if (setsockopt(sockfd, IPPROTO_IP, IP_ADD_MEMBERSHIP, (char *)&mgroup, sizeof(mgroup)) < 0) {
			perror("setsockopt() IPPROTO_IP: error ");
			close(sockfd);
			return 0;
    }		

 
   // quick hack, if i put a -X in the dev number, it means this is a EagleII IT951X device for num X
    if (argv[3][0] == '-') {
      handleType = EAGLEII; // EagleII == IT951X
      argv[3][0] = '0'; // remove the minus
    } else {
      handleType = EAGLEI; // Eagle == IT950X
    } 
    DevNo = atoi(argv[3]); // number device /dev/usb-it950x0 (last digit)

        /* Open handle Power On and Check device information */
        if((dwError = g_ITEAPI_TxDeviceInit(handleType, DevNo)) == ERR_NO_ERROR)
                printf("g_ITEAPI_TxDeviceInit ok\n");
        else
                printf("g_ITEAPI_TxDeviceInit fail\n");

        if((dwError = g_ITEAPI_TxGetNumOfDevice(&Tx_NumOfDev, DevNo)) == ERR_NO_ERROR)
                printf("%d Devices\n", Tx_NumOfDev);
        else
                printf("g_ITEAPI_TxGetNumOfDevice error\n");

    
    if((dwError = g_ITEAPI_TxGetChipType(&ChipType, DevNo)) != ERR_NO_ERROR) {
			fprintf(stderr, "g_ITE_TxGetChipType fail\n");
			return 0;
    }

    if ((dwError = g_ITEAPI_TxGetDrvInfo(&driverInfo, DevNo))) {
			fprintf(stderr, "Get Driver Info failed 0x%lu!\n", dwError);
			return 0;
    } else {
			fprintf(stderr, "g_ITEAPI_GetDrvInfo ok\n");		
			fprintf(stderr, "DriverInfo.DriverVerion  = %s\n", driverInfo.DriverVerion);
			fprintf(stderr, "DriverInfo.APIVerion     = %s\n", driverInfo.APIVerion);
			fprintf(stderr, "DriverInfo.FWVerionLink  = %s\n", driverInfo.FWVerionLink);
			fprintf(stderr, "DriverInfo.FWVerionOFDM  = %s\n", driverInfo.FWVerionOFDM);
			fprintf(stderr, "DriverInfo.Company       = %s\n", driverInfo.Company);
			fprintf(stderr, "DriverInfo.SupportHWInfo = %s\n", driverInfo.SupportHWInfo);
			fprintf(stderr, "DriverInfo.ProductID     = 0x%x\n", ChipType);
    }

    if ((dwError = g_ITEAPI_TxSetChannel(Freq, Bw, DevNo)) != ERR_NO_ERROR) {
			fprintf(stderr, "g_ITEAPI_TxSetChannel error\n");
			return 0;
    }
		
		// set parameters
		ChannelModulation_Setting.constellation = 2; // QPSK=0, QAM16=1, QAM64=2
		ChannelModulation_Setting.highCodeRate = 2; // 1/2 | 2/3 | 3/4 | 5/6 | 7/8 fec
		ChannelModulation_Setting.interval = 0; // 1/32 | 1/16 | 1/8 | 1/4 guard interval
		

    if (Bw == 2000) {
			fprintf(stderr, "Only 2k supported for 2MHz\n");
			ChannelModulation_Setting.transmissionMode = 0; // 0 = 2K
    } else {
      ChannelModulation_Setting.transmissionMode = 1; // 1 = 8K
    }
	
    fprintf(stderr, "Frequency = %lu KHz\n", Freq);
    fprintf(stderr, "Bandwidth = %d KHz\n", Bw);

	
    if ((dwError = g_ITEAPI_TxSetChannelModulation(ChannelModulation_Setting, DevNo)) != ERR_NO_ERROR)
	    fprintf(stderr, "g_ITEAPI_TxSetChannelModulation error!!, %lu ******\n", dwError);
	
	
	

    ChannelCapacity = Bw * 1000;
    ChannelCapacity = ChannelCapacity * ( ChannelModulation_Setting.constellation * 2 + 2);

    switch (ChannelModulation_Setting.interval) {
			case 0: //1/32
				ChannelCapacity = ChannelCapacity*32/33;
				break;
			case 1: //1/16
				ChannelCapacity = ChannelCapacity*16/17;
				break;
			case 2: //1/8
				ChannelCapacity = ChannelCapacity*8/9;
				break;
			case 3: //1/4
				ChannelCapacity = ChannelCapacity*4/5;
				break;
    }
    switch (ChannelModulation_Setting.highCodeRate) {
			case 0: //1/2
				ChannelCapacity = ChannelCapacity*1/2;
				break;
			case 1: //2/3
				ChannelCapacity = ChannelCapacity*2/3;
				break;
			case 2: //3/4
				ChannelCapacity = ChannelCapacity*3/4;
				break;
			case 3: //5/6
				ChannelCapacity = ChannelCapacity*5/6;
				break;
			case 4: //7/8
				ChannelCapacity = ChannelCapacity*7/8;
				break;
    }

    ChannelCapacity = ChannelCapacity/544*423;
    fprintf(stderr, "Channel Capacity is %lu bps\n",ChannelCapacity);

    g_ITEAPI_TxGetGainRange(Freq, Bw, &MaxGain, &MinGain, DevNo);
    fprintf(stderr, "MinGain: %d, MaxGain: %d\n", MinGain, MaxGain);

    if(g_ITEAPI_TxAdjustOutputGain(adjustoutputgain, &outgainvalue, DevNo) != ERR_NO_ERROR) {
	    fprintf(stderr, "g_ITEAPI_TxAdjustOutputGain error\n");
	    return 0;
    }



    completed = 0;
    g_ITEAPI_StartTransfer(DevNo);

    while(!completed) {
			len = recvfrom(sockfd, udp_packet, UDP_MAXIMUM_SIZE, 0, (struct sockaddr *) &addr,&addrlen);
			g_ITEAPI_TxSendTSData(udp_packet, len, DevNo); // send to device
      if(len<0){
        //completed=1;
        perror("send_data: error");
      }
    }
		
    g_ITEAPI_StopTransfer(DevNo);
    g_ITEAPI_TxDeviceExit(DevNo);
    return 0;
}
